from abc import ABC, abstractmethod

class Word(ABC):
    @abstractmethod
    def get_word(self):
        pass

    @abstractmethod
    def get_translation(self):
        pass

    @abstractmethod
    def get_example(self):
        pass



from .regular_verb import RegularVerb

class ARRegularVerb(RegularVerb):
    def __init__(self, word, translation):
        super().__init__(word, translation)
        self.ending = 'ar'

    def get_conjugation(self, pronoun):
        if pronoun == 'yo':
            return self.base + 'o'
        elif pronoun == 'tu':
            return self.base + 'as'
        elif pronoun == 'el':
            return self.base + 'a'
        elif pronoun == 'nosotros':
            return self.base + 'amos'
        elif pronoun == 'vosotros':
            return self.base + 'ais'
        elif pronoun == 'ellos':
            return self.base + 'an'
        else:
            return 'Unknown pronoun'
        
    def get_translation(self):
        return self.translation
    
    def get_example(self):
        pass


from .base_db import BaseDB
import os
from ..nouns import Noun

class NounsDB(BaseDB):
    def __init__(self):
        """Initialize the NounsDB class with a predefined database file and data."""
        super().__init__("nouns.db")
        self.initialize_database()
        

    def initialize_database(self):
        """Initialize the database with predefined tables if it doesn't exist."""
        if not os.path.exists(self.db_file):
            self.create_connection()
            self.create_predefined_table()
            self.close_connection()
        else:
            # Ensure the connection is established if the database file already exists
            self.create_connection()

    def create_predefined_table(self):
        """Create the unified table in the database."""
        create_nouns_table_query = """
        CREATE TABLE IF NOT EXISTS nouns (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            noun TEXT NOT NULL,
            translation TEXT NOT NULL
        );
        """
        self.execute_query(create_nouns_table_query)

    def insert_nouns(self, nouns : list[Noun]):
        """Insert nouns into the unified table."""
        insert_query = "INSERT INTO nouns (noun, translation) VALUES (?, ?);"
        for noun in nouns:
            self.execute_query(insert_query, (noun.word, noun.translation))

    def get_all_nouns(self):
        """Retrieve all nouns from the database."""
        select_nouns_query = "SELECT * FROM nouns;"
        return self.execute_read_query(select_nouns_query)




import datetime
import json
from .verbs import RegularVerb
from .nouns import Noun
from .abstract.word import Word
from .db import VerbsDB, NounsDB 
import questionary
import random
import unicodedata

class Quiz():

    def __init__(self):
        self.tracking = {}
        self.asked_questions = set()

    def _select_unique_word(self, word_list, cls):
        word = None
        while word is None or word.word in self.asked_questions:
            word = Noun.db_factory(random.choice(word_list)) if cls == Noun else RegularVerb.db_factory(random.choice(word_list))
        self.asked_questions.add(word.word)
        return word

    def _ask_question(self, word, word_type, language='random'):
        # Ask for the English translation
        if language == 'en':
            correct = self.ask_english_question(word, word_type)
        elif language == 'es':
            correct = self.ask_spanish_question(word, word_type)
        else:
            # get random language
            if random.choice([True,False]):
                return self._ask_question(word, word_type, 'en')
            else:
                return self._ask_question(word, word_type, 'es')

        if correct:
            print("Correct!")
        else:
            if language == 'en':
                print(f"Wrong! The correct English translation is: {word.translation}")
            else :
                print(f"Wrong! The correct Spanish translation is: {word.word}")

    def ask_spanish_question(self, word, word_type):
        question = f"What is the Spanish translation of the English {word_type}: {word.translation}?"
        answer = questionary.text(question).ask()
        correct = self._compare_with_or_without_accents(answer, word.word)
        self.tracking[question] = {'given_answer': answer, 'correct_answer': word.word, 'correct': correct}
        return correct
    
    def ask_english_question(self, word, word_type):
        question = f"What is the English translation of the Spanish {word_type}: {word.word}?"
        answer = questionary.text(question).ask()
        correct = self._compare_with_or_without_accents(answer, word.translation)
        self.tracking[question] = {'given_answer': answer, 'correct_answer': word.translation, 'correct': correct}
        return correct
    
    def _compare_with_or_without_accents(self, str1, str2):
        normalized_str1 = unicodedata.normalize('NFKD', str1).casefold()
        normalized_str2 = unicodedata.normalize('NFKD', str2).casefold()
        
        stripped_str1 = ''.join(c for c in normalized_str1 if not unicodedata.combining(c))
        stripped_str2 = ''.join(c for c in normalized_str2 if not unicodedata.combining(c))
        
        return stripped_str1 == stripped_str2

    def test_regular_verb(self, number_of_questions):
        verbs = VerbsDB().get_all_verbs()
        for _ in range(number_of_questions):
            verb : RegularVerb = self._select_unique_word(verbs, RegularVerb)
            self._ask_question(verb, "verb")

    def test_noun(self, number_of_questions):
        nouns = NounsDB().get_all_nouns()
        for _ in range(number_of_questions):
            noun : Noun = self._select_unique_word(nouns, Noun)
            self._ask_question(noun, "noun")

    def test(self, number_of_questions):
        self.test_regular_verb(number_of_questions)
        self.test_noun(number_of_questions)
        
    def get_tracking(self):
        return self.tracking
    
    def tracking_to_csv(self):
        import csv
        date = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        with open(f'aprendo_quiz_{date}.csv', 'w') as f:
            writer = csv.writer(f)
            writer.writerow(['Question', 'Given Answer', 'Correct Answer', 'Correct'])
            for key, value in self.tracking.items():
                writer.writerow([key, value['given_answer'], value['correct_answer'], value['correct']])
    
    def print_tracking(self) :
        print(json.dumps(self.get_tracking(), indent=4))




from .base_db import BaseDB
import os
from ..verbs import RegularVerb

class VerbsDB(BaseDB):
    def __init__(self):
        """Initialize the VerbsDB class with a predefined database file and data."""
        super().__init__("regular_verbs.db")
        print('init called - verbs db')
        self.initialize_database()
        

    def initialize_database(self):
        """Initialize the database with predefined tables if it doesn't exist."""
        if not os.path.exists(self.db_file):
            print('initialize database called')
            self.create_connection()
            self.create_predefined_table()
            self.close_connection()
        else:
            # Ensure the connection is established if the database file already exists
            self.create_connection()

    def create_predefined_table(self):
        """Create the unified table in the database."""
        create_verbs_table_query = """
        CREATE TABLE IF NOT EXISTS verbs (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            verb TEXT NOT NULL,
            translation TEXT NOT NULL,
            ending TEXT NOT NULL
        );
        """
        self.execute_query(create_verbs_table_query)

    def insert_verbs(self, verbs : list[RegularVerb]):
        """Insert verbs into the unified table."""
        insert_query = "INSERT INTO verbs (verb, translation, ending) VALUES (?, ?, ?);"
        for verb in verbs:
            self.execute_query(insert_query, (verb.word, verb.translation, verb.get_ending()))

    def get_all_verbs(self):
        """Retrieve all verbs from the database."""
        select_verbs_query = "SELECT * FROM verbs;"
        return self.execute_read_query(select_verbs_query)

    def get_verbs_by_ending(self, ending):
        """Retrieve verbs by their ending."""
        select_verbs_query = "SELECT * FROM verbs WHERE ending = ?;"
        return self.execute_read_query(select_verbs_query, (ending,))



from ..abstract.word import Word
from abc import ABC, abstractmethod
from .constants import SPANISH_PRONOUNS

class Verb(Word, ABC) :
    def __init__(self, word, translation):
        self.word = word
        self.translation = translation
        self.pronouns = SPANISH_PRONOUNS

    def get_word(self):
        return self.word

    @abstractmethod
    def get_translation(self):
        return self.translation

    @abstractmethod
    def get_example(self):
        pass

    @abstractmethod
    def get_conjugation(self):
        pass

   


from .verb import Verb


class RegularVerb(Verb):
    def __init__(self, word, translation):
        super().__init__(word, translation)
        self.base = self.remove_stem()

    @classmethod
    def factory(cls, word, translation):
        from .er_regular_verb import ERRegularVerb
        from .ir_regular_verb import IRRegularVerb
        from .ar_regular_verb import ARRegularVerb

        if word.endswith('ar'):
            return ARRegularVerb(word, translation)
        elif word.endswith('er'):
            return ERRegularVerb(word, translation)
        elif word.endswith('ir'):
            return IRRegularVerb(word, translation)
        else:
            return None
        
    def remove_stem(self):
        return self.word[:-2]
    
    def get_ending(self):
        return self.word[-2:]
    
    def db_factory(db_row):
        word = db_row[1]
        translation = db_row[2]
        return RegularVerb.factory(word, translation)



import sqlite3
from sqlite3 import Error

class BaseDB:
    def __init__(self, db_file):
        """Initialize the BaseDB class with the database file."""
        self.db_file = db_file
        self.connection = None

    def create_connection(self):
        """Create a database connection to the SQLite database specified by db_file."""
        try:
            self.connection = sqlite3.connect(self.db_file)
            print(f"Connection to {self.db_file} established.")
        except Error as e:
            print(f"Error connecting to database: {e}")

    def execute_query(self, query, params=None):
        """Execute a single query."""
        if self.connection is None:
            print("Connection is not established.")
            return

        try:
            cursor = self.connection.cursor()
            if params:
                cursor.execute(query, params)
            else:
                cursor.execute(query)
            self.connection.commit()
            print("Query executed successfully.")
        except Error as e:
            print(f"Error executing query: {e}")

    def execute_read_query(self, query, params=None):
        """Execute a read query and return the results."""
        if self.connection is None:
            print("Connection is not established.")
            return

        try:
            cursor = self.connection.cursor()
            if params:
                cursor.execute(query, params)
            else:
                cursor.execute(query)
            result = cursor.fetchall()
            return result
        except Error as e:
            print(f"Error reading data: {e}")

    def close_connection(self):
        """Close the database connection."""
        if self.connection:
            self.connection.close()
            print(f"Connection to {self.db_file} closed.")



from ..abstract.word import Word

class Noun(Word):
    def __init__(self, word : str, translation : str):
        self.word = word
        self.translation = translation

    def __repr__(self):
        return f"{self.word} - {self.translation}"
    
    def db_factory(db_row):
        word = db_row[1]
        translation = db_row[2]
        return Noun(word, translation)
    
    def get_word(self):
        return self.word
    
    def get_translation(self):
        return self.translation
    
    def get_example(self):
        pass


from setuptools import setup, find_packages
from pathlib import Path
import subprocess

this_dir = Path(__file__).parent
long_description = (this_dir / "README.md").read_text()

def get_version_from_git():
    try:
        # Get the latest tag from Git
        version = subprocess.check_output(['git', 'describe', '--tags']).strip().decode('utf-8')
        # Ensure the version is in the correct format
        if version.startswith('v'):
            version = version[1:]
        return version
    except subprocess.CalledProcessError:
        # Handle the case where there are no tags or git is not available
        return "0.0.0"

setup(
    name='aprendo',
    version=get_version_from_git(),
    long_description=long_description,
    long_description_content_type='text/markdown',
    description='Quieres aprender español? "Aprendo" es la solución! Add, edit, conjugate, learn and test your Spanish verbs and vocab.',
    author='Lui Holliday',
    author_email='tech.luiholl@gmail.com',
    url='https://github.com/luiHoll97/aprendo',
    packages=find_packages(),
    install_requires=['questionary'],
)